import { ObservableValue, unobserved } from "./Observable";
import { makePropertyObservable } from "./ObservableObject";
/** _Property decorator_, makes a property observable on every instance, with a read-only value shared across all instances taken from an (earlier OR later) call to `inject`, *until* the property is directly assigned to [requires ES5+ target] [decorator] */
export function injectable(target, key, descriptor) {
    if (!descriptor)
        descriptor = { enumerable: true };
    // capture existing value (e.g. method on prototype)
    var value = descriptor.value;
    delete descriptor.value;
    delete descriptor.writable;
    // use a single observable per class to capture the last injected value
    var injector = ObservableValue.fromValue(value);
    descriptor.get = function () {
        var _this = this;
        unobserved(function () {
            makePropertyObservable(_this, key, ObservableValue.fromValue(injector));
        });
        return this[key];
    };
    // setter just initializes an observable with given value
    // (will ignore injection altogether)
    descriptor.set = function (value) {
        makePropertyObservable(this, key);
        this[key] = value;
    };
    // store a reference to the observable injector value
    descriptor.get["*injector"] = injector;
    return descriptor;
}
/** Set given injectable observable properties (decorated with `injectable`) on *all* instances of given class and derived classes to the given values, except for instances where the property has been overwritten directly; may be called multiple times even with the same properties to redefine their injected value; returns an object that contains the previous injected values (for e.g. overriding an injected function that calls into the previously injected function; when injecting into a derived class, these properties use accessors to return latest overridden injected values on base class(es) dynamically) */
export function inject(targetClass, spec) {
    var result = {};
    var _loop_1 = function () {
        if (!Object.prototype.hasOwnProperty.call(spec, name))
            return "continue";
        // get the injector observable for the property on given prototype,
        // add a value or getter on the result object that reflects the old OR
        // overridden (observable) value
        var getInjector = function (proto, d) {
            if (!proto)
                return undefined;
            var desc = Object.getOwnPropertyDescriptor(proto, name);
            if (!desc || !desc.get || !desc.get["*injector"]) {
                // recurse if not defined at this level
                var getter_1 = function () { return getInjector(Object.getPrototypeOf(proto)); };
                var injector_1 = getter_1();
                if (d && injector_1) {
                    // found the injector further down the prototype chain
                    d.get = function () { return getter_1().getLastValue(); };
                }
                return injector_1;
            }
            else {
                // found the injector observable
                var injector_2 = desc.get["*injector"];
                if (d) {
                    // store the last value
                    d.value = injector_2.getLastValue();
                }
                return injector_2;
            }
        };
        d = { enumerable: true, configurable: false };
        injector = getInjector(targetClass.prototype, d);
        if (!injector)
            throw new TypeError("Not an injected property: " + name);
        // store getter or value on result object
        Object.defineProperty(result, name, d);
        // if injecting on derived class, use overriding injector instead
        if (d.get) {
            override = injectable(targetClass.prototype, name);
            Object.defineProperty(targetClass.prototype, name, override);
            injector = override.get["*injector"];
        }
        // set the new value in the (new) injector observable
        injector.value = spec[name];
    };
    var d, injector, override;
    for (var name in spec) {
        _loop_1();
    }
    return result;
}
