import { Promise } from "./Promise";
/** Encapsulates a handler connected to a signal */
export interface SignalConnection {
    /** Disconnect the connected handler from this signal */
    disconnect(): void;
    /** True if connection has been disconnected */
    disconnected: boolean;
}
/** Encapsulates a signal that can be used to trigger one or more handlers asynchronously; do _not_ construct `Signal` classes directly, use `defineSignal` to create derived classes which can be instantiated and emitted */
export declare abstract class Signal<T> {
    /** [implementation] Add a handler */
    static connect(callback: any, target?: {}): SignalConnection;
    /** [implementation] Add a one-time handler */
    static connectOnce(callback: any, target?: {}): void;
    /** [implementation] Remove all handlers */
    static disconnectAll(): typeof Signal;
    /** [implementation] Returns true if this signal has any handlers */
    static isConnected(): boolean;
    /** Static method that is called synchronously when a first handler is connected (and when a new handler is connected after all had been disconnected); override this in a signal base class, e.g. to add a delayed initialization method */
    protected static onHandlerConnected?: () => void;
    /** Static method that is called synchronously when no more handlers are connected; override this in a signal base class, e.g. to add a delayed deallocation method */
    protected static onHandlersDisconnected?: () => void;
    /** Create a signal with given payload data, ready to be emitted */
    constructor(data: T);
    /** Invoke all handlers and capture promises of their return values in .results (unless argument is true); works only once; returns this */
    emit(noResults?: boolean): this;
    /** Invoke given callback(s) with return values of all handlers, or any exception that occurred during execution of all handlers; returns a Promise that resolves to the return value of the callback(s) itself */
    then<O>(onFulfilled?: (results: any[]) => O, onRejected?: (error: Error) => O): Promise<O>;
    /** Catch errors that occur during execution of all handlers; returns a Promise that resolves to the return value of the callback itself */
    catch<O>(onRejected?: (error: Error) => O): Promise<O>;
    /** Array of Promises that resolve to return values of all handlers */
    readonly results: Promise<any>[];
    /** The payload data for this signal instance */
    readonly data: T;
}
export declare namespace Signal {
    /** Type definition for a callable (emittable) signal *class*; matches the result of `defineSignal`; the type parameters represent handler function arguments: signal payload, and emitted signal type */
    interface Emittable<T, SignalClassT extends typeof Signal> {
        /** Instantiate a signal with given value, ready to be emitted */
        new (data: T): Signal<T>;
        /** Emit a signal with given value; does not instantiate this class unless there are actually handlers connected to it */
        (data?: T): void;
        /** Add a handler to be invoked when this signal is emitted; returns an encapsulation of the connection with a disconnect method */
        connect(handler: (data: T, source: SignalClassT) => any): SignalConnection;
        /** Add a handler to be invoked when this signal is emitted: a method with given name on given target object (resolved only when needed); returns an encapsulation of the connection with a disconnect method */
        connect(method: string, target: {}): SignalConnection;
        /** Add a handler to be invoked the next time this signal is emitted */
        connectOnce(handler: (data: T, source: SignalClassT) => any): void;
        /** Add a handler to be invoked the next time this signal is emitted: a method with given name on given target object (resolved only when needed) */
        connectOnce(method: string, target: {}): void;
        /** Remove all handlers */
        disconnectAll(): this;
        /** Returns true if this signal has any handlers connected to it */
        isConnected(): boolean;
    }
}
/** Create a new emittable signal class that derives directly from `Signal`, with given payload type */
export declare function defineSignal<T>(): Signal.Emittable<T, typeof Signal>;
/** Create a new emittable signal class that derives from given base class, which must derive from `Signal`, and set given (static) properties on the created class */
export declare function defineSignal<SignalT extends typeof Signal, T>(Base: SignalT & {
    new (...args): {
        data: T;
    };
}, properties?: {}): Signal.Emittable<T, SignalT>;
/** Signal that is triggered with exceptions that were unhandled during async execution; a custom handler may be added here, the default handler just writes a warning message to the console; to disable this behavior for specific errors, set error.message to a blank string */
export declare const UnhandledException: Signal.Emittable<Error, typeof Signal>;
